"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true,
});
exports.Bot = void 0;

const errors_1 = require("../errors");

const messageMapper_1 = require("../mapper/messageMapper");

const utils_1 = require("../utils");

class Bot {
  constructor(bot) {
    this._bot = bot;
  }

  get voiceState() {
    return this._bot.voiceConnection;
  }

  get isLoggedIn() {
    return this._bot.isLoggedIn();
  }

  isMessageAuthor(message) {
    return message.author.id === this._bot.id;
  }

  async joinVoiceChannel(channelId) {
    return await this._bot.joinVoiceChannel(channelId);
  }

  leaveVoiceChannel() {
    this._bot.leaveVoiceChannel();
  }

  isInVoiceChannel() {
    return this._bot.isInVoiceChannel();
  }

  async fetchChannel(id) {
    return await this._bot.fetchChannel(id);
  }

  async fetchGuild(id) {
    return await this._bot.fetchGuild(id);
  }

  getChannel() {
    return this._bot.channel;
  }

  getGuild() {
    return this._bot.guild;
  }

  findGuild(id) {
    return this._bot.findGuild(id);
  }

  async send(message) {
    if (!message) {
      throw new Error("Can not send a empty message");
    }

    if (!this._bot.isLoggedIn()) {
      throw new errors_1.CordeClientError(
        "Can not send a directly message to channel because the client is not connected yet",
      );
    }

    if (utils_1.isPrimitiveValue(message)) {
      return await this._bot.sendMessage(message);
    }

    const embed = messageMapper_1.mapper.embedInterfaceToMessageEmbed(message);
    return await this._bot.sendMessage(embed);
  }

  async createRole(data) {
    if (!this._bot.isLoggedIn()) {
      throw new errors_1.CordeClientError("Bot is not connected yet. Can not create a role");
    }

    try {
      return await this._bot.roleManager.create({
        data,
      });
    } catch (error) {
      throw Error("Could not create the role. " + error);
    }
  }

  findRole(data) {
    if (!this._bot.isLoggedIn()) {
      throw new errors_1.CordeClientError("Bot is not connected yet. No role can be searched");
    }

    return this._findRole(data);
  }

  _findRole(data) {
    if (typeof data === "string") {
      return this._bot.getRoles().find((r) => r.id === data);
    }

    return this._bot.getRoles().find((r) => r.id === data.id || r.name === data.name);
  }
}

exports.Bot = Bot;
